@@ -1,15 +1,44 @@
 """Module where the `on_page_markdown` plugin event is processed."""
 
+from __future__ import annotations
+
 import glob
 import html
 import logging
 import os
 import re
-import string
 import textwrap
+from collections.abc import MutableMapping
+from typing import TYPE_CHECKING, Any
 
 from mkdocs_include_markdown_plugin import process
-from mkdocs_include_markdown_plugin.config import CONFIG_DEFAULTS
+from mkdocs_include_markdown_plugin.config import (
+    CONFIG_DEFAULTS,
+    DEFAULT_CLOSING_TAG,
+    DEFAULT_COMMENTS,
+    DEFAULT_OPENING_TAG,
+    create_include_tag,
+)
+from mkdocs_include_markdown_plugin.files_watcher import FilesWatcher
+from mkdocs_include_markdown_plugin.regexes import (
+    DOUBLE_QUOTED_STR_RE,
+    SINGLE_QUOTED_STR_RE,
+)
+
+
+if TYPE_CHECKING:  # remove this for mypyc compiling
+    import sys
+
+    if sys.version_info >= (3, 8):
+        from typing import TypedDict
+    else:
+        from typing_extensions import TypedDict
+
+    from mkdocs.structure.pages import Page
+
+    class DirectiveBoolArgument(TypedDict):  # noqa: D101
+        value: bool
+        regex: re.Pattern[str]
 
 
 logger = logging.getLogger('mkdocs.plugins.mkdocs_include_markdown_plugin')
@@ -24,46 +53,20 @@ TRUE_FALSE_BOOL_STR = {
     False: 'false',
 }
 
-BOOL_ARGUMENT_PATTERN = r'\w+'
-DOUBLE_QUOTED_STR_ARGUMENT_PATTERN = r'([^"]|(?<=\\)["])+'
-SINGLE_QUOTED_STR_ARGUMENT_PATTERN = r"([^']|(?<=\\)['])+"
 
-# In the following regexp, he substrings "$OPENING_TAG" and "$CLOSING_TAG"
-# will be replaced by the effective opening and closing tags in the
-# `on_page_markdown` method below.
-INCLUDE_TAG_REGEX = re.compile(
-    rf"""
-        (?P<_includer_indent>[ \t\f\v\w{re.escape(string.punctuation)}]*?)$OPENING_TAG
-        \s*
-        include
-        \s+
-        (?:"(?P<double_quoted_filename>{DOUBLE_QUOTED_STR_ARGUMENT_PATTERN})")?(?:'(?P<single_quoted_filename>{SINGLE_QUOTED_STR_ARGUMENT_PATTERN})')?
-        (?P<arguments>.*?)
-        \s*
-        $CLOSING_TAG
-    """,  # noqa: E501
-    flags=re.VERBOSE | re.DOTALL,
-)
-
-INCLUDE_MARKDOWN_TAG_REGEX = re.compile(
-    INCLUDE_TAG_REGEX.pattern.replace(' include', ' include-markdown'),
-    flags=INCLUDE_TAG_REGEX.flags,
-)
+def bool_arg(arg: str) -> re.Pattern[str]:
+    """Return a compiled regexp to match a boolean argument."""
+    return re.compile(rf'{arg}=(\w+)')
 
 
-def str_arg(arg):
+def str_arg(arg: str) -> re.Pattern[str]:
     """Return a compiled regexp to match a string argument."""
     return re.compile(
-        rf'{arg}=(?:"({DOUBLE_QUOTED_STR_ARGUMENT_PATTERN})")?'
-        rf"(?:'({SINGLE_QUOTED_STR_ARGUMENT_PATTERN})')?",
+        rf'{arg}=(?:"({DOUBLE_QUOTED_STR_RE})")?'
+        rf"(?:'({SINGLE_QUOTED_STR_RE})')?",
     )
 
 
-def bool_arg(arg):
-    """Return a compiled regexp to match a boolean argument."""
-    return re.compile(rf'{arg}=({BOOL_ARGUMENT_PATTERN})')
-
-
 ARGUMENT_REGEXES = {
     'start': str_arg('start'),
     'end': str_arg('end'),
@@ -82,7 +85,9 @@ ARGUMENT_REGEXES = {
 }
 
 
-def parse_filename_argument(match):
+def parse_filename_argument(
+        match: re.Match[str],
+) -> tuple[str | None, str | None]:
     """Return the filename argument matched by ``match``."""
     raw_filename = match.group('double_quoted_filename')
     if raw_filename is None:
@@ -96,7 +101,7 @@ def parse_filename_argument(match):
     return filename, raw_filename
 
 
-def parse_string_argument(match):
+def parse_string_argument(match: re.Match[str]) -> str | None:
     """Return the string argument matched by ``match``."""
     value = match.group(1)
     if value is None:
@@ -108,33 +113,33 @@ def parse_string_argument(match):
     return value
 
 
-def lineno_from_content_start(content, start):
+def lineno_from_content_start(content: str, start: int) -> int:
     """Return the line number of the first line of ``start`` in ``content``."""
     return content[:start].count('\n') + 1
 
 
-def read_file(file_path, encoding):
+def read_file(file_path: str, encoding: str) -> str:
     """Read a file and return its content."""
     with open(file_path, encoding=encoding) as f:
         return f.read()
 
 
 def get_file_content(
-    markdown,
-    page_src_path,
-    docs_dir,
-    include_tag_regex,
-    include_markdown_tag_regex,
-    default_encoding,
-    default_preserve_includer_indent,
-    default_dedent,
-    default_trailing_newlines,
-    default_comments=CONFIG_DEFAULTS['comments'],
-    cumulative_heading_offset=0,
-    build=None,
-):
+    markdown: str,
+    page_src_path: str,
+    docs_dir: str,
+    include_tag_regex: re.Pattern[str],
+    include_markdown_tag_regex: re.Pattern[str],
+    default_encoding: str,
+    default_preserve_includer_indent: bool,
+    default_dedent: bool,
+    default_trailing_newlines: bool,
+    default_comments: bool = DEFAULT_COMMENTS,
+    cumulative_heading_offset: int = 0,
+    files_watcher: FilesWatcher | None = None,
+) -> str:
     """Return the content of the file to include."""
-    def found_include_tag(match):
+    def found_include_tag(match: re.Match[str]) -> str:
         directive_match_start = match.start()
 
         _includer_indent = match.group('_includer_indent')
@@ -162,12 +167,9 @@ def get_file_content(
                 filename,
             )
 
-        exclude_match = re.search(
-            ARGUMENT_REGEXES['exclude'],
-            arguments_string,
-        )
+        exclude_match = ARGUMENT_REGEXES['exclude'].search(arguments_string)
         if exclude_match is None:
-            ignore_paths = []
+            ignore_paths: list[str] = []
         else:
             exclude_string = parse_string_argument(exclude_match)
             if exclude_string is None:
@@ -209,10 +211,10 @@ def get_file_content(
                 f':{lineno}',
             )
             return ''
-        elif build is not None:
-            build.included_files.extend(file_paths_to_include)
+        elif files_watcher is not None:
+            files_watcher.included_files.extend(file_paths_to_include)
 
-        bool_options = {
+        bool_options: dict[str, DirectiveBoolArgument] = {
             'preserve-includer-indent': {
                 'value': default_preserve_includer_indent,
                 'regex': ARGUMENT_REGEXES['preserve-includer-indent'],
@@ -228,12 +230,14 @@ def get_file_content(
         }
 
         for arg_name, arg in bool_options.items():
-            match = re.search(arg['regex'], arguments_string)
-            if match is None:
+            bool_arg_match = arg['regex'].search(arguments_string)
+            if bool_arg_match is None:
                 continue
             try:
                 bool_options[arg_name]['value'] = TRUE_FALSE_STR_BOOL[
-                    match.group(1) or TRUE_FALSE_BOOL_STR[arg['value']]
+                    bool_arg_match.group(
+                        1,
+                    ) or TRUE_FALSE_BOOL_STR[arg['value']]
                 ]
             except KeyError:
                 lineno = lineno_from_content_start(
@@ -247,7 +251,7 @@ def get_file_content(
                 )
                 return ''
 
-        start_match = re.search(ARGUMENT_REGEXES['start'], arguments_string)
+        start_match = ARGUMENT_REGEXES['start'].search(arguments_string)
         if start_match:
             start = parse_string_argument(start_match)
             if start is None:
@@ -262,7 +266,7 @@ def get_file_content(
         else:
             start = None
 
-        end_match = re.search(ARGUMENT_REGEXES['end'], arguments_string)
+        end_match = ARGUMENT_REGEXES['end'].search(arguments_string)
         if end_match:
             end = parse_string_argument(end_match)
             if end is None:
@@ -277,10 +281,7 @@ def get_file_content(
         else:
             end = None
 
-        encoding_match = re.search(
-            ARGUMENT_REGEXES['encoding'],
-            arguments_string,
-        )
+        encoding_match = ARGUMENT_REGEXES['encoding'].search(arguments_string)
         if encoding_match:
             encoding = parse_string_argument(encoding_match)
             if encoding is None:
@@ -293,6 +294,7 @@ def get_file_content(
                     ' directive at '
                     f'{os.path.relpath(page_src_path, docs_dir)}:{lineno}',
                 )
+                encoding = 'utf-8'
         else:
             encoding = default_encoding
 
@@ -324,7 +326,7 @@ def get_file_content(
                 default_preserve_includer_indent,
                 default_dedent,
                 default_trailing_newlines,
-                build=build,
+                files_watcher=files_watcher,
             )
 
             # trailing newlines right stripping
@@ -372,7 +374,7 @@ def get_file_content(
 
         return text_to_include
 
-    def found_include_markdown_tag(match):
+    def found_include_markdown_tag(match: re.Match[str]) -> str:
         directive_match_start = match.start()
 
         _includer_indent = match.group('_includer_indent')
@@ -401,12 +403,9 @@ def get_file_content(
                 filename,
             )
 
-        exclude_match = re.search(
-            ARGUMENT_REGEXES['exclude'],
-            arguments_string,
-        )
+        exclude_match = ARGUMENT_REGEXES['exclude'].search(arguments_string)
         if exclude_match is None:
-            ignore_paths = []
+            ignore_paths: list[str] = []
         else:
             exclude_string = parse_string_argument(exclude_match)
             if exclude_string is None:
@@ -448,10 +447,10 @@ def get_file_content(
                 f':{lineno}',
             )
             return ''
-        elif build is not None:
-            build.included_files.extend(file_paths_to_include)
+        elif files_watcher is not None:
+            files_watcher.included_files.extend(file_paths_to_include)
 
-        bool_options = {
+        bool_options: dict[str, DirectiveBoolArgument] = {
             'rewrite-relative-urls': {
                 'value': True,
                 'regex': ARGUMENT_REGEXES['rewrite-relative-urls'],
@@ -475,12 +474,14 @@ def get_file_content(
         }
 
         for arg_name, arg in bool_options.items():
-            match = re.search(arg['regex'], arguments_string)
-            if match is None:
+            bool_arg_match = arg['regex'].search(arguments_string)
+            if bool_arg_match is None:
                 continue
             try:
                 bool_options[arg_name]['value'] = TRUE_FALSE_STR_BOOL[
-                    match.group(1) or TRUE_FALSE_BOOL_STR[arg['value']]
+                    bool_arg_match.group(
+                        1,
+                    ) or TRUE_FALSE_BOOL_STR[arg['value']]
                 ]
             except KeyError:
                 lineno = lineno_from_content_start(
@@ -496,7 +497,7 @@ def get_file_content(
                 return ''
 
         # start and end arguments
-        start_match = re.search(ARGUMENT_REGEXES['start'], arguments_string)
+        start_match = ARGUMENT_REGEXES['start'].search(arguments_string)
         if start_match:
             start = parse_string_argument(start_match)
             if start is None:
@@ -512,7 +513,7 @@ def get_file_content(
         else:
             start = None
 
-        end_match = re.search(ARGUMENT_REGEXES['end'], arguments_string)
+        end_match = ARGUMENT_REGEXES['end'].search(arguments_string)
         if end_match:
             end = parse_string_argument(end_match)
             if end is None:
@@ -528,10 +529,7 @@ def get_file_content(
         else:
             end = None
 
-        encoding_match = re.search(
-            ARGUMENT_REGEXES['encoding'],
-            arguments_string,
-        )
+        encoding_match = ARGUMENT_REGEXES['encoding'].search(arguments_string)
         if encoding_match:
             encoding = parse_string_argument(encoding_match)
             if encoding is None:
@@ -544,13 +542,13 @@ def get_file_content(
                     ' directive at '
                     f'{os.path.relpath(page_src_path, docs_dir)}:{lineno}',
                 )
+                encoding = 'utf-8'
         else:
             encoding = default_encoding
 
         # heading offset
         offset = 0
-        offset_match = re.search(
-            ARGUMENT_REGEXES['heading-offset'],
+        offset_match = ARGUMENT_REGEXES['heading-offset'].search(
             arguments_string,
         )
         if offset_match:
@@ -598,7 +596,7 @@ def get_file_content(
                 default_dedent,
                 default_trailing_newlines,
                 default_comments=default_comments,
-                build=build,
+                files_watcher=files_watcher,
             )
 
             # trailing newlines right stripping
@@ -672,13 +670,11 @@ def get_file_content(
 
         return text_to_include
 
-    markdown = re.sub(
-        include_tag_regex,
+    markdown = include_tag_regex.sub(
         found_include_tag,
         markdown,
     )
-    markdown = re.sub(
-        include_markdown_tag_regex,
+    markdown = include_markdown_tag_regex.sub(
         found_include_markdown_tag,
         markdown,
     )
@@ -686,46 +682,35 @@ def get_file_content(
 
 
 def on_page_markdown(
-    markdown,
-    page,
-    docs_dir,
-    config=None,
-    build=None,
-):
+    markdown: str,
+    page: Page,
+    docs_dir: str,
+    config: MutableMapping[str, Any] | None = None,
+    files_watcher: FilesWatcher | None = None,
+) -> str:
     """Process markdown content of a page."""
     if config is None:
         config = {}
 
-    escaped_opening_tag = re.escape(
-        config.get('opening_tag', CONFIG_DEFAULTS['opening_tag']),
-    )
-    escaped_closing_tag = re.escape(
-        config.get('closing_tag', CONFIG_DEFAULTS['closing_tag']),
-    )
-
-    # Replace the substrings OPENING_TAG and CLOSING_TAG from
-    # INCLUDE_TAG_REGEX and INCLUDE_MARKDOWN_TAG_REGEX by the
-    # effective tags
-    include_tag_regex = re.compile(
-        INCLUDE_TAG_REGEX.pattern.replace(
-            '$OPENING_TAG', escaped_opening_tag,
-        ).replace('$CLOSING_TAG', escaped_closing_tag),
-        flags=INCLUDE_TAG_REGEX.flags,
-    )
-
-    include_markdown_tag_regex = re.compile(
-        INCLUDE_MARKDOWN_TAG_REGEX.pattern.replace(
-            '$OPENING_TAG', escaped_opening_tag,
-        ).replace('$CLOSING_TAG', escaped_closing_tag),
-        flags=INCLUDE_MARKDOWN_TAG_REGEX.flags,
-    )
-
     return get_file_content(
         markdown,
         page.file.abs_src_path,
         docs_dir,
-        include_tag_regex,
-        include_markdown_tag_regex,
+        config.get(
+            '_include_tag',
+            create_include_tag(
+                config.get('opening_tag', DEFAULT_OPENING_TAG),
+                config.get('closing_tag', DEFAULT_CLOSING_TAG),
+            ),
+        ),
+        config.get(
+            '_include_markdown_tag',
+            create_include_tag(
+                config.get('opening_tag', DEFAULT_OPENING_TAG),
+                config.get('closing_tag', DEFAULT_CLOSING_TAG),
+                tag='include-markdown',
+            ),
+        ),
         config.get('encoding', CONFIG_DEFAULTS['encoding']),
         config.get(
             'preserve_includer_indent',
@@ -734,5 +719,5 @@ def on_page_markdown(
         config.get('dedent', CONFIG_DEFAULTS['dedent']),
         config.get('trailing_newlines', CONFIG_DEFAULTS['trailing_newlines']),
         default_comments=config.get('comments', CONFIG_DEFAULTS['comments']),
-        build=build,
+        files_watcher=files_watcher,
     )
