@@ -55,21 +55,24 @@ class IncludeMarkdownPlugin(BasePlugin):
 
     def _watch_included_files(self) -> None:
         global FILES_WATCHER, SERVER
-        FILES_WATCHER = cast(FilesWatcher, FILES_WATCHER)
         SERVER = cast(LiveReloadServer, SERVER)
 
-        # unwatch previous watched files not needed anymore
-        for filepath in FILES_WATCHER.prev_included_files:
-            if filepath not in FILES_WATCHER.included_files:
-                SERVER.unwatch(filepath)
-        FILES_WATCHER.prev_included_files = (
-            FILES_WATCHER.included_files[:]
-        )
+        # compatibility with Mkdocs < 1.4.0
+        if hasattr(SERVER, 'unwatch'):
+            FILES_WATCHER = cast(FilesWatcher, FILES_WATCHER)
 
-        # watch new included files
-        for filepath in FILES_WATCHER.included_files:
-            SERVER.watch(filepath, recursive=False)
-        FILES_WATCHER.included_files = []
+            # unwatch previous watched files not needed anymore
+            for filepath in FILES_WATCHER.prev_included_files:
+                if filepath not in FILES_WATCHER.included_files:
+                    SERVER.unwatch(filepath)
+            FILES_WATCHER.prev_included_files = (
+                FILES_WATCHER.included_files[:]
+            )
+
+            # watch new included files
+            for filepath in FILES_WATCHER.included_files:
+                SERVER.watch(filepath, recursive=False)
+            FILES_WATCHER.included_files = []
 
     def on_page_content(
             self,
